@@ -25,6 +25,7 @@
 
 #include <glib.h>
 #include <glib/gi18n.h>
+#include <glib/gstdio.h>
 #include <stdio.h>
 #include <string.h>
 #include <math.h>
@@ -78,6 +79,159 @@ typedef struct
 
 } BgClosure;
 
+static time_t
+get_mtime (const char *filename)
+{
+        GFile     *file;
+        GFileInfo *info;
+        time_t     mtime;
+
+        mtime = (time_t)-1;
+
+        if (filename) {
+                file = g_file_new_for_path (filename);
+                info = g_file_query_info (file, G_FILE_ATTRIBUTE_TIME_MODIFIED,
+                                          G_FILE_QUERY_INFO_NONE, NULL, NULL);
+                if (info) {
+                        mtime = g_file_info_get_attribute_uint64 (info,
+                                                                  G_FILE_ATTRIBUTE_TIME_MODIFIED);
+                        g_object_unref (info);
+                }
+                g_object_unref (file);
+        }
+
+        return mtime;
+}
+
+/*
+ * Add the caching code used in gnome-desktop as well (but too difficult to bind
+ * with clutter interaction
+ *
+ */
+static char*
+get_wallpaper_cache_dir ()
+{
+  return g_build_filename (g_get_user_cache_dir(), "wallpaper", NULL);
+}
+
+static char *
+get_wallpaper_cache_prefix_name (const char        *placement,
+                                int                width,
+                                int                height)
+{
+  return g_strdup_printf ("%s_%i_%i", placement, width, height);
+}
+
+static char *
+get_wallpaper_cache_filename (const char        *placement,
+                              const char        *filename,
+                              int                width,
+                              int                height)
+{
+  gchar *cache_basename;
+  gchar *cache_filename;
+  gchar *cache_dir;
+  gchar *cache_prefix_name;
+
+  cache_prefix_name = get_wallpaper_cache_prefix_name (placement, width, height);
+  cache_basename = g_strdup_printf ("%s_%s", cache_prefix_name, filename);
+  cache_basename = g_strdelimit (cache_basename, G_DIR_SEPARATOR_S, '_');
+  cache_dir = get_wallpaper_cache_dir ();
+  cache_filename = g_build_filename (cache_dir, cache_basename, NULL);
+
+  g_free (cache_prefix_name);
+  g_free (cache_basename);
+  g_free (cache_dir);
+
+  return cache_filename;
+
+}
+
+static gboolean
+cache_file_is_valid (const char *filename,
+                    const char *cache_filename)
+{
+  time_t mtime = get_mtime (filename);
+  time_t cache_mtime = get_mtime (cache_filename);
+
+  if (!g_file_test (cache_filename, G_FILE_TEST_IS_REGULAR))
+     return FALSE;
+  return (mtime < cache_mtime);
+}
+
+static void
+refresh_cache_file (const char  *placement,
+                    const char  *filename,
+                    GdkPixbuf   *new_pixbuf,
+                    int          width,
+                    int          height)
+{
+  gchar *cache_filename;
+  gchar *cache_dir;
+  GdkPixbufFormat *format;
+  gchar* format_name;
+  GDir* g_cache_dir;
+  const gchar* file;
+  gchar *cache_prefix_name;
+
+  cache_prefix_name = get_wallpaper_cache_prefix_name (placement, width, height);
+  cache_filename = get_wallpaper_cache_filename (placement, filename, width, height);
+  cache_dir = get_wallpaper_cache_dir ();
+
+  /* Only refresh scaled file on disk if useful (and don't cache slideshow) */
+  if (!cache_file_is_valid (filename, cache_filename)) {
+     format = gdk_pixbuf_get_file_info (filename, NULL, NULL);
+     if (format != NULL) {
+        if (!g_file_test (cache_dir, G_FILE_TEST_IS_DIR))
+           g_mkdir (cache_dir, 0700);
+        else {
+           g_cache_dir = g_dir_open (cache_dir, 0, NULL);
+           file = g_dir_read_name (g_cache_dir);
+           gchar *path;
+           while (file != NULL) {
+              path = g_build_filename (cache_dir, file, NULL);
+              /* only purge files with same transformation and resolution */
+              if (strstr (file, cache_prefix_name) && g_file_test (path, G_FILE_TEST_IS_REGULAR))
+                 g_unlink (path);
+              file = g_dir_read_name (g_cache_dir);
+              g_free (path);
+           }
+           g_dir_close (g_cache_dir);
+        }
+        format_name = gdk_pixbuf_format_get_name (format);
+        if (strcmp (format_name, "jpeg") == 0)
+           gdk_pixbuf_save (new_pixbuf, cache_filename, format_name, NULL, "quality", "100", NULL);
+        else
+           gdk_pixbuf_save (new_pixbuf, cache_filename, format_name, NULL, NULL);
+        g_free (format_name);
+     } 
+  }
+
+  g_free (cache_prefix_name);
+  g_free(cache_filename);
+  g_free(cache_dir);
+}
+
+static GdkPixbuf *
+load_from_cache_file (const char *placement,
+                      const char *filename,
+                      int         best_width,
+                      int         best_height)
+{
+  GdkPixbuf *pixbuf = NULL;
+  gchar *cache_filename;
+
+  cache_filename = get_wallpaper_cache_filename (placement, filename, best_width, best_height);
+  if (cache_file_is_valid (filename, cache_filename))
+     pixbuf = gdk_pixbuf_new_from_file (cache_filename, NULL);
+  g_free (cache_filename);
+
+  return pixbuf;
+}
+
+
+
+
 /*
  * Just load the file normally,if its larger than the stage, clip it
  */
@@ -89,7 +243,10 @@ load_normal (const gchar *filename, ClutterActor *texture)
   gint width, height;
   gint subw = 0, subh = 0;
 
-  pixbuf = gdk_pixbuf_new_from_file (filename, &error);
+  // try cache first
+  pixbuf = load_from_cache_file("none", filename, CSW () -1, CSH ());
+  if (!pixbuf)
+    pixbuf = gdk_pixbuf_new_from_file (filename, &error);
 
   if (error)
     {
@@ -137,7 +294,10 @@ load_stretched (const gchar *filename, ClutterActor *texture)
   GdkPixbuf *pixbuf = NULL;
   GError *error = NULL;
 
-  pixbuf = gdk_pixbuf_new_from_file_at_scale (filename,
+  // try cache first
+  pixbuf = load_from_cache_file("stretched", filename, CSW () -1, CSH ());
+  if (!pixbuf)
+    pixbuf = gdk_pixbuf_new_from_file_at_scale (filename,
            CSW (),
            CSH (),
            FALSE,
@@ -167,7 +327,10 @@ load_centred (const gchar *filename, ClutterActor *texture)
   GError *error = NULL;
   gint w, h, x, y;
 
-  pixbuf = gdk_pixbuf_new_from_file (filename, &error);
+  // try cache first
+  pixbuf = load_from_cache_file("centred", filename, CSW () -1, CSH ());
+  if (!pixbuf)
+    pixbuf = gdk_pixbuf_new_from_file (filename, &error);
 
   if (error)
     {
@@ -199,7 +362,10 @@ load_scaled (const gchar *filename, ClutterActor *texture)
   GError *error = NULL;
   gint w, h, x, y;
 
-  pixbuf = gdk_pixbuf_new_from_file_at_scale (filename,
+  // try cache first
+  pixbuf = load_from_cache_file("scaled", filename, CSW () -1, CSH ());
+  if (!pixbuf)
+    pixbuf = gdk_pixbuf_new_from_file_at_scale (filename,
            CSW (),
            CSH (),
            TRUE,
@@ -238,7 +404,10 @@ load_zoomed (const gchar *filename, ClutterActor *texture)
   gint current_width, current_height;
   gdouble factor, factor_height;
 
-  pixbuf = gdk_pixbuf_new_from_file (filename,
+  // try cache first
+  pixbuf = load_from_cache_file("zoom", filename, CSW () -1, CSH ());
+  if (!pixbuf)
+    pixbuf = gdk_pixbuf_new_from_file (filename,
                                      &error);
   if (error)
     {
@@ -285,7 +454,10 @@ load_wallpaper (const gchar *filename, ClutterActor *texture)
   gint w, h, x, y;
   gint rows, cols, r, c;
 
-  pixbuf = gdk_pixbuf_new_from_file (filename, &error);
+  // try cache first
+  pixbuf = load_from_cache_file("wallpaper", filename, CSW () -1, CSH ());
+  if (!pixbuf)
+    pixbuf = gdk_pixbuf_new_from_file (filename, &error);
 
   if (error)
     {
@@ -382,6 +554,7 @@ ensure_layout (NlBackground *bg)
     {
       pixbuf = load_stretched (priv->filename, priv->texture);
     }
+  refresh_cache_file (priv->option, priv->filename, pixbuf, CSW () -1, CSH ());
 
   clutter_actor_queue_redraw (CLUTTER_ACTOR (bg));
 }
