@@ -36,7 +36,7 @@
 #include "nl-window.h"
 #include "nl-quit.h"
 
-G_DEFINE_TYPE (NlFavoriteView, nl_favorite_view, CTK_TYPE_ACTOR);
+G_DEFINE_TYPE (NlFavoriteView, nl_favorite_view, CTK_TYPE_BIN);
 
 #define NL_FAVORITE_VIEW_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE \
         ((obj), NL_TYPE_FAVORITE_VIEW, NlFavoriteViewPrivate))
@@ -417,7 +417,7 @@ nl_favorite_view_init (NlFavoriteView *self)
   view = priv->view = ctk_icon_view_new ();
   ctk_actor_set_padding (CTK_ACTOR (view), &vpadding);
   ctk_actor_set_background_for_state (CTK_ACTOR (view), CTK_STATE_NORMAL, bg);
-  clutter_actor_set_parent (view, CLUTTER_ACTOR (self));
+  clutter_container_add_actor (CLUTTER_CONTAINER (self), view);
   clutter_actor_set_opacity (view, 0);
   clutter_actor_show (view);
 }
