@@ -382,7 +382,8 @@ on_folders_changed (NlFoldersSource *source)
     }
   add_launcher_to_view (source, desktop);
 
-  if ((desktop_uri && g_strcmp0  (desktop_uri, download_uri) != 0) || !desktop_uri)
+  if (((desktop_uri && g_strcmp0 (desktop_uri, download_uri) != 0) || !desktop_uri) &&
+      ((home_uri && g_strcmp0 (home_uri, download_uri) != 0) || !home_uri))
     {
      if (!download && download_uri)
        {
