@@ -520,14 +520,15 @@ on_clicked (NlSidebarItem *item, NlShell *shell)
     {
       LauncherApplication *app = a->data;
       ClutterActor *tile, *emblem;
-      GdkPixbuf *pixbuf;
+      GdkPixbuf *pixbuf = NULL;
 
       if (!a)
         continue;
 
-      pixbuf = nl_pixbuf_cache_icon_for_name (items->priv->cache,
-                                      launcher_application_get_icon_name (app),
-                                              64);
+      if (launcher_application_get_icon_name (app))
+        pixbuf = nl_pixbuf_cache_icon_for_name (items->priv->cache,
+                                                launcher_application_get_icon_name (app),
+                                                64);
       if (!pixbuf)
         pixbuf = nl_pixbuf_cache_icon_for_name (items->priv->cache,
                                                 "application-x-executable",
@@ -573,6 +574,14 @@ on_child_focus_changed (ClutterActor    *icon_view,
   gfloat y1 = 0, y2 = 0;
   ClutterActorBox focus_box = { 0 };
 
+  /* For some reason somewhere between signal emission and this handler being
+   * called, box can get NULLed (eg: LP #514083).
+   * 
+   * I think this problem should be hunted down at some point.  Until then
+   * work around it by checking for NULL.
+   */
+  g_return_if_fail (box != NULL);
+
   parent = clutter_actor_get_parent (icon_view);
 
   children = clutter_container_get_children (main_box);
@@ -628,7 +637,8 @@ on_system_clicked (NlSidebarItem *item, NlMenuItems *items)
       gchar            *markup;
       GSList           *a;
 
-      if (!g_strstr_len (launcher_category_get_icon_name (cat),
+      if (!launcher_category_get_icon_name (cat) ||
+          !g_strstr_len (launcher_category_get_icon_name (cat),
                          -1,
                          "preferences"))
         continue;
@@ -663,14 +673,15 @@ on_system_clicked (NlSidebarItem *item, NlMenuItems *items)
         {
           LauncherApplication *app = a->data;
           ClutterActor *tile, *emblem;
-          GdkPixbuf *pixbuf;
+          GdkPixbuf *pixbuf = NULL;
 
           if (!a)
             continue;
 
-          pixbuf = nl_pixbuf_cache_icon_for_name (items->priv->cache,
-                                      launcher_application_get_icon_name (app),
-                                                 64);
+          if (launcher_application_get_icon_name (app))
+            pixbuf = nl_pixbuf_cache_icon_for_name (items->priv->cache,
+                                                    launcher_application_get_icon_name (app),
+                                                    64);
           tile = nl_icon_tile_new (launcher_application_get_name (app),
                                    launcher_application_get_comment (app),
                                    pixbuf);
@@ -737,13 +748,14 @@ on_menu_changed (LauncherMenu *menu, NlMenuItems *items)
     {
       LauncherCategory *category = c->data;
       ClutterActor *item;
-      GdkPixbuf *pixbuf;
+      GdkPixbuf *pixbuf = NULL;
       gchar     *icon_name;
 
       if (!category)
         continue;
 
-      if (g_strstr_len (launcher_category_get_icon_name (category),
+      if (launcher_category_get_icon_name (category) &&
+          g_strstr_len (launcher_category_get_icon_name (category),
                         -1,
                         "preferences"))
         {
@@ -751,19 +763,22 @@ on_menu_changed (LauncherMenu *menu, NlMenuItems *items)
           continue;
         }
 
-      icon_name = g_strconcat ("unr-",
-                               launcher_category_get_icon_name (category),
-                               NULL);
-      pixbuf = nl_pixbuf_cache_icon_for_name (priv->cache,
-                                              icon_name,
-                                              48);
-      if (!pixbuf)
-        {
+      if (launcher_category_get_icon_name (category))
+        { 
+          icon_name = g_strconcat ("unr-",
+                                   launcher_category_get_icon_name (category),
+                                   NULL);
           pixbuf = nl_pixbuf_cache_icon_for_name (priv->cache,
-                                 launcher_category_get_icon_name (category),
-                                 48);
+                                                  icon_name,
+                                                  48);
+          if (!pixbuf)
+            {
+              pixbuf = nl_pixbuf_cache_icon_for_name (priv->cache,
+                                                      launcher_category_get_icon_name (category),
+                                                      48);
+            }
+          g_free (icon_name);
         }
-      g_free (icon_name);
 
       item = nl_sidebar_item_new (launcher_category_get_name (category),
                                   launcher_category_get_comment (category),
